#!/bin/bash

# Install ClamAV with SELinux support

# If we haven't read functions.sh by now, then read it
if [ "x$ZTFUNCTIONS" = "x" ]; then
  if [ -f lib/functions.sh ]; then
    . lib/functions.sh
  elif [ -f ../lib/functions.sh ]; then
    cd ..
    . lib/functions.sh
  else
    echo 'Sorry, I need to be able to source functions.sh first.'
    echo Exiting...
    exit 1
  fi
fi

shout
shout =================================================================
shout
shout Install and configure ClamAV
shout
shout =================================================================
shout
pause

shout Installing ClamAV packages
apt-get -y install clamav clamav-daemon

# Change default SelfCheck time from 1h to 10mins.
shout Making clamd notice new signatures much faster
sed -i.zendto '/^SelfCheck/ s/[0-9][0-9]*/600/' /etc/clamav/clamd.conf

shout Stop freshclam daemon so we can update signatures
if [ "$OS" = "debian" -o "$OSVER" -ge "16" ]; then
  canrun systemctl && systemctl stop clamav-freshclam
else
  /etc/init.d/clamav-freshclam stop
fi
shout Updating signatures  
shout '1. Ignore errors about not being able to notify clamd'
shout '2. Expect a delay after downloading main.cvd'
shout '3. Expect a long delay after downloading daily.cvd'
freshclam

shout Allowing ClamAV to read Apache files
usermod -a -G www-data clamav
# and the other way around!
usermod -a -G clamav www-data

shout Allowing ClamAV through AppArmor to read ZendTo uploads
if [ -d /etc/apparmor.d -a -f /etc/init.d/apparmor ]; then
  mkdir -p /etc/apparmor.d/local # Just in case
  APP=/etc/apparmor.d/local/usr.sbin.clamd
  # Create it if necessary
  if [ ! -f $APP ]; then
    :> $APP
    chown root:root $APP
    chmod 0644 $APP
  fi
  if fgrep -q zendto $APP; then
    shout No need, already done.
  else
    echo '# ZendTo settings (do not delete this line)' >> $APP
    echo '/var/zendto/** r,' >> $APP
    echo 'capability chown,' >> $APP
  fi

  # Fix bug in Ubuntu 18 apparmor profile of clamd
  if [ "$OS" = "ubuntu" -a "$OSVER" -ge "18" ]; then
    REPAIRCLAMD=yes
    shout 'Ubuntu 18: may need to work around a bug in the clamd AppArmor profile'
    CLAM=/etc/apparmor.d/usr.sbin.clamd
    # This gives us either "" or "flags=(blah blah)"
    FLAGS="$( grep '^/usr/sbin/clamd\s*' "$CLAM" | head -1 | sed -e 's/^\/usr\/sbin\/clamd\s*//; s/\s*{.*$//' )"
    if [[ $FLAGS =~ flags ]]; then
      # There are already some flags. Do they include the one we need?
      if [[ $FLAGS =~ attach_disconnected ]]; then
        shout 'Good, they have fixed it. No change needed'
        REPAIRCLAMD=no
      else
        # No, so add to the list of flags
        shout 'Adding the "attach_disconnected" flag to workaround Ubuntu 18 bug'
        FLAGS="$( echo "$FLAGS" | sed -e 's/)/ attach_disconnected)/' )"
      fi
    else
      # No flags present at all, so add new one
      shout 'Adding the "attach_disconnected" flag to workaround Ubuntu 18 bug'
      FLAGS="flags=(attach_disconnected)"
    fi
    if [ "x$REPAIRCLAMD" = "xyes" ]; then
      # Replace any current flags with our new expanded list
      shout "Modifying the flags in the existing $CLAM"
      sed -i 's/^\(\/usr\/sbin\/clamd\)\s*.*$/\1 '"$FLAGS"' {/' "$CLAM"
    fi
  fi

  # Debian 10 and recent Ubuntus have aa-teardown instead of apparmor teardown
  if canrun aa-teardown; then
    aa-teardown
  else
    /etc/init.d/apparmor teardown
  fi
  apparmor_parser --purge-cache
  /etc/init.d/apparmor start
  /etc/init.d/apparmor reload
else
  shout "You do not appear to have AppArmor installed."
fi

# Do we need to patch the clamd systemd settings to disable PrivateTmp?
# If so, start by finding the service definition file for "apache2"
SFILE="$( canrun systemctl && systemctl show -p FragmentPath apache2.service | sed -e 's/^.*= *//' | head -1)"
if [ "x$SFILE" != "x" -a -e "$SFILE" ]; then
  # Is there "PrivateTmp=true" in the [Service] section of it?
  cat /lib/systemd/system/apache2.service | \
  sed -n -e '/^\[Service\]/,/^$/p' | \
  grep -qi '^PrivateTmp *= *true'
  if [ "$?" = "0" ]; then
    # Yes, PrivateTmp=true, so override it correctly.
    OVERDIR=/etc/systemd/system/apache2.service.d
    mkdir -p "$OVERDIR"
    if grep -q '^PrivateTmp *= *false' "$OVERDIR"/*.conf 2>/dev/null; then
      # Don't have to do anything as it's already set to false
      :
    else
      OVERFILE="${OVERDIR}/PrivateTmp_zendto.conf"
      shout "I am going to have to override the systemd definition for"
      shout "apache2 so it starts up with PrivateTmp=false."
      shout "I am creating $OVERFILE"
      pause
      cat <<EOAPAC > "$OVERFILE"
#
# ZendTo: Apache runs PHP which runs clamdscan.
# If Apache is running with a private /tmp then clamdscan fails.
#
[Service]
PrivateTmp=false
EOAPAC
      shout "And tell systemd about the change"
      pause
      systemctl daemon-reload
    fi
  fi
fi


shout Starting ClamAV and freshclam daemons
if [ "$OS" = "debian" -o "$OSVER" -ge "16" ]; then
  canrun systemctl && systemctl stop clamav-freshclam
  canrun systemctl && systemctl restart clamav-daemon
  canrun systemctl && systemctl start clamav-freshclam
else
  /etc/init.d/clamav-freshclam stop
  /etc/init.d/clamav-daemon restart
  /etc/init.d/clamav-freshclam start
fi

shout
shout ClamAV has been setup for you to work with ZendTo.
shout

exit 0

